#!/usr/bin/perl -w

use strict;
use Getopt::Long;

#Translation stuff
my %encTrans = ( "BINARY_CONDITION"=>"bc", "VECTOR_CONDITION"=>"vc", "CHAR_CONDITION"=>"cc" );
my %rstTrans = ( "RULESET_LIST"=>"rsl" );
my %archTrans = ( "ARCH_SSE2"=>"sse2", "ARCH_ALTIVEC"=>"altivec" );

#Output dir
my $resDir="res";

#Parameters

# Process options.
my $seed=69;
my $numCond=100;
my $numRules=5000;
my $numIns=2000;
my $numExp=50;

my $encoding = "VECTOR_CONDITION"; # CHAR_CONDITION BINARY_CONDITION
my $arch = "ARCH_SSE2"; #"ARCH_ALTIVEC";
my $ruleSetType = "RULESET_LIST";

if ( @ARGV > 0 ) {
GetOptions('seed=i'=> \$seed,
	   'number_conditions=i' => \$numCond,
	   'number_rules=i' => \$numRules,
	   'number_instances=i' => \$numIns,
	   'number_experiments=i' => \$numExp,
	   'encoding=s' => \$encoding,
	   'architecture=s' => \$arch,
	   'rule_set_type=s' => \$ruleSetType,
	   'output_dir=s' => \$resDir);
}
else {
	print "Proper parameters not provided\n\n";
	print "\t--seed: Random seed to use\n";
	print "\t--number_conditions: Number of conditions to generate\n";
	print "\t--number_rules: Number of rules to generate\n";
	print "\t--number_instances: Number of instances to generate\n";
	print "\t--number_experiments: Number of experiments to run\n";
	print "\t--encoding: Encoding to use (VECTOR_CONDITION|CHAR_CONDITION|BINARY_CONDITION)\n";
	print "\t--architecture: Architecture for vector operations (ARCH_SSE2|ARCH_ALTIVEC)\n";
	print "\t--rule_set_type: Type of rulse set to use (RULESET_LIST)\n";
	print "\t--output_dir: The output dir for the results\n";

	exit 0;
}

my $att = ""; 
if ( $encoding eq "VECTOR_CONDITION" ) {
	$att = "-$archTrans{$arch}";
}

my $extFlag = "";
if ( $arch eq "ARCH_ALTIVEC") {
	$extFlag =" -faltivec";
}

#File names
my $binaryFileName = "fmlt.run-$encTrans{$encoding}".$att."-$rstTrans{$ruleSetType}-$seed-$numCond-$numRules-$numIns-$numExp.bin";
my $resultFileName = "fmlt.res-$encTrans{$encoding}".$att."-$rstTrans{$ruleSetType}-$seed-$numCond-$numRules-$numIns-$numExp.txt";

#Compiling the experiment
print "\t\tCompiling...";
`gcc -O3 $extFlag -D$encoding -D$arch -D$ruleSetType -DRANDOM_SEED=$seed -DNUM_CONDITIONS=$numCond -DNUM_RULES=$numRules -DNUM_INSTANCES=$numIns -DNUM_EXPERIMENTS=$numExp -o $binaryFileName -I.:src/ src/*.c`;
print "done.\n";

#Running the experiment
print "\t\tRunning...";
`./$binaryFileName > ./$resDir/$resultFileName`;
print "done.\n";

1;
