/*
 * 
 *  Copyright (C) Xavier Llor
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 */
/*
 *  definitions.h
 *  LCSMatching
 *
 *  Contains the basis alternatives for the implementation of rule conditions
 *
 */
 
#ifndef _DEFINITIONS_
#define _DEFINITIONS_

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>

// 
// Condition definition ruleset
//
#ifdef VECTOR_CONDITION
// A vector based implementation
#ifdef ARCH_SSE2
#define CONDITION_IMPLEMENTATION "binary sse2 vector"
#else
#ifdef ARCH_ALTIVEC
#define CONDITION_IMPLEMENTATION "binary altivec vector"
#else
#define CONDITION_IMPLEMENTATION "binary unknown vector"
#endif
#endif

#else
#ifdef BINARY_CONDITION
// A word based binary implementation
#define CONDITION_IMPLEMENTATION "binary word"
#else
// The default character implementation 
#define CHARACTER_CONDITION
#define CONDITION_IMPLEMENTATION "character encoding"
#endif 
#endif

//
// The ruleset implementation
//
#ifdef RULESET_LIST
// An indexed list based implementation
#define RULESET_IMPLEMENTATION "list of rules"
#else
// The default implementation
#define RULESET_LIST
#define RULESET_IMPLEMENTATION "list of rules"
#endif


//
// Number of conditions
//
#ifndef NUM_CONDITIONS
#define NUM_CONDITIONS 11
#endif

//
// Default random seed
//
#ifndef NUM_INSTANCES
#define NUM_INSTANCES 1000
#endif


//
// Number of rules
//
#ifndef NUM_RULES
#define NUM_RULES 1000
#endif

//
// Default random seed
//
#ifndef RANDOM_SEED
#define RANDOM_SEED 69
#endif

//
// Default number of experiments
//
#ifndef NUM_EXPERIMENTS
#define NUM_EXPERIMENTS 10
#endif


#endif
 


