/*
 * 
 *  Copyright (C) Xavier Llor
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 */
/*
 *  instance.h
 *  LCSMatching
 *
 *  Contains the instance type declaration.
 *
 */
 
#ifndef _INSTANCE_
#define _INSTANCE_
 
#include "definitions.h" 

#ifdef VECTOR_CONDITION
// A vector based implementation
#define INSTANCE unsigned int*

#define RECODE_BLOCKS (4*(2*NUM_CONDITIONS/(8*4*sizeof(unsigned int))+1))
#define ATT_PER_BLOCK ((8*sizeof(unsigned int))/2)
#define NEW_INSTANCE ((unsigned int *)malloc(RECODE_BLOCKS*sizeof(unsigned int)))
#define DELETE_INSTANCE(A) free(A)

#define ATT_MASK 0x3U 
#define ALL_MASK ((unsigned int)-1)
#define GET_ATT(INS,IDX) ((INS&(ATT_MASK<<(2*IDX)))>>(2*IDX))
#define SET_ATT(INS,VAL,IDX) (INS=(((ALL_MASK^(ATT_MASK<<(2*IDX)))&INS)|(VAL<<(2*IDX))))

#define GET_ATT_IDX(INS,IDX) GET_ATT(INS[IDX/ATT_PER_BLOCK],IDX%ATT_PER_BLOCK)
#define SET_ATT_IDX(INS,VAL,IDX) SET_ATT(INS[IDX/ATT_PER_BLOCK],VAL,IDX%ATT_PER_BLOCK)

#define RANDOM_INIT_INSTANCE(INS) {\
		int i,j;\
		unsigned int tmp,tmpAtt;\
		for(i=0,j=0; i<=RECODE_BLOCKS ; i++) {\
			tmp = (unsigned int)isprng();\
			for (j=0; j<ATT_PER_BLOCK ; j++){\
				if ( (i*ATT_PER_BLOCK+j)<NUM_CONDITIONS ) {\
					tmpAtt = GET_ATT(tmp,j);\
					switch (tmpAtt) {\
						case 1:\
						case 2: break;\
						default: ((isprng()%2)?SET_ATT(tmp,1,j):SET_ATT(tmp,2,j)); break;\
					}\
				}\
				else\
					SET_ATT(tmp,0,j);\
			}\
			INS[i] = tmp;\
		}\
	}

#define PRINT_INSTANCE(STREAM,INS) {\
	int i,j,tmp;\
	for (i=0;i<=RECODE_BLOCKS;i++){\
		for (j=0;j<ATT_PER_BLOCK && (i*ATT_PER_BLOCK+j)<NUM_CONDITIONS;j++){\
			tmp = GET_ATT(INS[i],j);\
			switch (tmp) {\
				case 1: fprintf(STREAM,"0"); break;\
				case 2: fprintf(STREAM,"1"); break;\
				default: fprintf(STREAM,"?"); break;\
			}\
		}\
	}\
}


#else
#ifdef BINARY_CONDITION
// A word based binary implementation
#define INSTANCE unsigned int*

#define RECODE_BLOCKS (2*NUM_CONDITIONS/(8*sizeof(unsigned int))+1)
#define ATT_PER_BLOCK ((8*sizeof(unsigned int))/2)
#define NEW_INSTANCE ((unsigned int *)malloc(RECODE_BLOCKS*sizeof(unsigned int)))
#define DELETE_INSTANCE(A) free(A)

#define ATT_MASK 0x3U 
#define ALL_MASK ((unsigned int)-1)
#define GET_ATT(INS,IDX) ((INS&(ATT_MASK<<(2*IDX)))>>(2*IDX))
#define SET_ATT(INS,VAL,IDX) (INS=(((ALL_MASK^(ATT_MASK<<(2*IDX)))&INS)|(VAL<<(2*IDX))))

#define GET_ATT_IDX(INS,IDX) GET_ATT(INS[IDX/ATT_PER_BLOCK],IDX%ATT_PER_BLOCK)
#define SET_ATT_IDX(INS,VAL,IDX) SET_ATT(INS[IDX/ATT_PER_BLOCK],VAL,IDX%ATT_PER_BLOCK)

#define RANDOM_INIT_INSTANCE(INS) {\
		int i,j;\
		unsigned int tmp,tmpAtt;\
		for(i=0,j=0; i<=RECODE_BLOCKS ; i++) {\
			tmp = (unsigned int)isprng();\
			for (j=0; j<ATT_PER_BLOCK ; j++){\
				if ( (i*ATT_PER_BLOCK+j)<NUM_CONDITIONS ) {\
					tmpAtt = GET_ATT(tmp,j);\
					switch (tmpAtt) {\
						case 1:\
						case 2: break;\
						default: ((isprng()%2)?SET_ATT(tmp,1,j):SET_ATT(tmp,2,j)); break;\
					}\
				}\
				else\
					SET_ATT(tmp,0,j);\
			}\
			INS[i] = tmp;\
		}\
	}


#define PRINT_INSTANCE(STREAM,INS) {\
	int i,j,tmp;\
	for (i=0;i<=RECODE_BLOCKS;i++){\
		for (j=0;j<ATT_PER_BLOCK && (i*ATT_PER_BLOCK+j)<NUM_CONDITIONS;j++){\
			tmp = GET_ATT(INS[i],j);\
			switch (tmp) {\
				case 1: fprintf(STREAM,"0"); break;\
				case 2: fprintf(STREAM,"1"); break;\
				default: fprintf(STREAM,"?"); break;\
			}\
		}\
	}\
}

#else
// The default character implementation 
#define CHAR_CONDITION
#define INSTANCE char*

#define GET_ATT_IDX(INS,IDX) (INS[IDX])
#define SET_ATT_IDX(INS,VAL,IDX) (INS[IDX]=VAL)

#define NEW_INSTANCE ((char *)malloc(NUM_CONDITIONS*sizeof(char)))
#define DELETE_INSTANCE(A) free(A)

#define RANDOM_INIT_INSTANCE(INS) {\
		int i;\
		for(i=0; i<NUM_CONDITIONS ; i++)\
			switch ( (isprng()%2UL) ) {\
				case  0UL: INS[i] = '0'; break;\
				case  1UL: INS[i] = '1'; break;\
			}\
	}
	
#define PRINT_INSTANCE(STREAM,INS) {\
	int i;\
	for (i=0;i<NUM_CONDITIONS;i++)\
		fprintf(STREAM,"%c",INS[i]);\
	}

#endif 
#endif

#endif

