/*
 * 
 *  Copyright (C) Xavier Llor
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 */
#include <stdio.h>
#include <time.h>
#include <sys/time.h>

/*
#define RANDOM_SEED 6239

#define NUM_CONDITIONS 128
#define NUM_RULES 5000
#define NUM_INSTANCES 2000
#define NUM_EXPERIMENTS 10

#define VECTOR_CONDITION
#define ARCH_ALTIVEC

#define RULESET_LIST
*/

#include "instance.h"
#include "ruleset.h"

int timeExperiment (int argc, const char * argv[]) {
	
    // Strating the whole thing
	// Few dumping of the configuration information for the test
	printf("Run information\n");
    printf("---------------\n");
    printf("[RI] Number of conditions: %d\n", NUM_CONDITIONS);
	printf("[RI] Number of rules: %d\n",NUM_RULES);
	printf("[RI] Condition implementation: %s\n",CONDITION_IMPLEMENTATION);
	printf("[RI] Rule set implementation: %s\n",RULESET_IMPLEMENTATION);
	printf("[RI] Numeber of experiments ot average: %d\n\n",NUM_EXPERIMENTS);
	
	// Initializing the pseudo-random number generator
	printf("Pseudo-random number generator\n");
    printf("------------------------------\n");
    printf("[MT] Initializing Mersenne Twister...\n");
	printf("[MT] Initial random seed: %d\n",RANDOM_SEED);
	init_Mersenne(RANDOM_SEED);
	printf("[MT] First random number generated: %u\n",isprng());
	printf("[MT] ...done\n\n");
   	
	// Initializing the rule set to us for the testing
	printf("Rule set to use\n");
    printf("---------------\n");
    printf("[RS] Initializing the rule set...\n");
	RULESET rs = NEW_RULESET;
	assert(rs);
	INIT_RULESET(rs);
	RANDOMIZE_RULESET(rs);
	printf("[RS] Rules generated: %d\n",NUM_RULES);
	printf("[RS] First rule of the set: ");
	PRINT_RULE(stdout,GET_RULE_FROM_RULESET(rs,0));
	printf("\n");
	printf("[RS] Last rule of the set:  ");
	PRINT_RULE(stdout,GET_RULE_FROM_RULESET(rs,NUM_RULES-1));
	printf("\n");
	printf("[RS] ...done\n");
	#ifdef RULESET_TREE
	printf("[RI] Building the rule set index...\n");
	RULESETINDEX rsi = buildMatchIndex(rs);
	printf("[RI] ...done\n\n");
	#endif
	#ifdef RULESET_FLATTREE
	printf("[RI] Building the rule set index...\n");
	RULESETINDEX rsi = buildMatchIndex(rs);
	printf("[RI] ...done\n\n");
	#endif

	// Initializing the perfornance unit
	printf("Environment generator\n");
    printf("---------------------\n");
    printf("[EG] Initializing instance generator...\n");
	printf("[EG] Number of instances to generate: %d\n",NUM_INSTANCES);
	INSTANCE ins = NEW_INSTANCE;
	RANDOM_INIT_INSTANCE(ins);
	printf("[EG] First instance generated: ");
	PRINT_INSTANCE(stdout,ins);
	printf("\n[EG] ...done\n");
	printf("\n");
	
	// The experiment control variables
	int iExpCnt,iInsCnt;
	
	// Initializing the time measures
	printf("Starting the time measures\n");
    printf("--------------------------\n");
    printf("[TM] Starting the time measure...\n");
	
	struct timeval time[2];
	long lDeltaTime;
	if (gettimeofday(&(time[0]), NULL) != 0) {
		printf("[ERROR] Could not initialize the time measure. Aborting\n" );
		exit(1);
	}
	
	// Executing the matching time expereiment measrures
	// The time measured experiment to use
	for ( iExpCnt=0 ; iExpCnt<NUM_EXPERIMENTS ; iExpCnt++ )
	{
		// The code to evaluate
		for ( iInsCnt=0 ; iInsCnt<NUM_INSTANCES ; iInsCnt++ ) {
			#ifdef RULESET_FLATTREE
			computeMatchSet(rsi,ins);
			#else
			#ifdef RULESET_TREE
			countMatchedRules(rsi,ins);
			#else
			countMatchedRules(rs,ins);
			#endif
			#endif
		}
		RANDOM_INIT_INSTANCE(ins);
	}
	
	// Computing the time spend ton the matching phase
	if (gettimeofday(&(time[1]), 0) != 0) {
		printf("[ERROR] Could not get the final time measure. Aborting\n" );
		exit(1);
	}
	
	lDeltaTime = ((long)time[1].tv_sec - (long)time[0].tv_sec)*1000000L+
				 ((long)time[1].tv_usec - (long)time[0].tv_usec);
	printf("[TM] Total time : %f secs\n",  ((double)lDeltaTime/1000000.0));  
	printf("[TM] Average time per experiment : %f secs\n",  ((double)lDeltaTime/1000000.0/NUM_EXPERIMENTS));  
	printf("[TM] ...done\n");
	 	
	
	// Exiting with no error
    return 0;
}

int main (int argc, const char * argv[]) {
	
	init_Mersenne(RANDOM_SEED);
	
	return timeExperiment(argc,argv);
}