/*
 * 
 *  Copyright (C) Xavier Llor
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 */
/*
 *  rule.h
 *  LCSMatching
 *
 *  Created by Little Bee on 12/10/05.
 *  Copyright 2005 __MyCompanyName__. All rights reserved.
 *
 */

#ifndef _RULE_
#define _RULE_
 
#include "definitions.h" 
#include "instance.h"

#ifdef VECTOR_CONDITION
// A vector based implementation

#ifdef ARCH_SSE2
#include <emmintrin.h>
#endif

#define RULE unsigned int*

#define NEW_RULE ((unsigned int *)malloc(RECODE_BLOCKS*sizeof(unsigned int)))
#define DELETE_RULE(A) free(A)

#define RANDOM_INIT_RULE(R) {\
		int i,j,tmpAtt;\
		for(i=0,j=0; i<=RECODE_BLOCKS ; i++) {\
			R[i] = (unsigned int)isprng();\
			for (j=0; j<ATT_PER_BLOCK ; j++){\
				if ( (i*ATT_PER_BLOCK+j)<NUM_CONDITIONS ) {\
					tmpAtt = GET_ATT(R[i],j);\
					switch (tmpAtt) {\
						case 0: SET_ATT(R[i],3,j); break;\
					}\
				}\
				else\
					SET_ATT(R[i],0,j);\
			}\
		}\
	}

#define PRINT_RULE(STREAM,R) {\
	int i,j,tmp;\
	for (i=0;i<=RECODE_BLOCKS;i++){\
		for (j=0;j<ATT_PER_BLOCK && (i*ATT_PER_BLOCK+j)<NUM_CONDITIONS;j++){\
			tmp = GET_ATT(R[i],j);\
			switch (tmp) {\
				case 1: fprintf(STREAM,"0"); break;\
				case 2: fprintf(STREAM,"1"); break;\
				default: fprintf(STREAM,"#"); break;\
			}\
		}\
	}\
}

int isRuleMatched ( RULE rule, INSTANCE ins ) {
	register int iFlag = 1;
	
	#ifdef ARCH_ALTIVEC
	// Matching code for altivec (Powerpc G4, G5)
	register int i,iMax,tmp;
	register vector unsigned int vir,vii;

	for ( i=0, iMax=RECODE_BLOCKS/4  ; i<iMax /*&& iFlag*/ ; i++) { 
			tmp = i*4;
			vir = vec_ld(0, &rule[tmp]);
			vii = vec_ld(0, &ins[tmp]);
			vir = vec_and(vir,vii);
			//iFlag = vec_all_eq(vir,vii);
			iFlag &= vec_all_eq(vir,vii);
	}
	
	#else
	#ifdef ARCH_SSE2
	// Matching code for SSE2 (Opteron and Xeon)
	register int i,iMax,tmp;
	__m128i vir,vii;

	for ( i=0, iMax=RECODE_BLOCKS/4  ; i<iMax /*&& iFlag*/ ; i++) { 
			tmp = i*4;
			vir = _mm_load_si128((__m128i*)&rule[tmp]);
			vii  = _mm_load_si128((__m128i*)&ins[tmp]);
			vir = _mm_and_si128(vir,vii);
			vii = _mm_cmpeq_epi32(vir,vii);
			//iFlag = (-1 == _mm_movemask_epi8(vii));
			iFlag &= (-1 == _mm_movemask_epi8(vii));
	}
	#else
	// Unespecified architecture (Abort execution)
	fprintf(stderr,"Vectorized matching requires Altivec or SSE2 defines!!!\n");
	abort();
	
	#endif
	#endif		
	
	return iFlag;
}


#else
#ifdef BINARY_CONDITION
// A word based binary implementation
#define RULE unsigned int*

#define NEW_RULE ((unsigned int *)malloc(RECODE_BLOCKS*sizeof(unsigned int)))
#define DELETE_RULE(A) free(A)

#define RANDOM_INIT_RULE(R) {\
		int i,j,tmpAtt;\
		for(i=0,j=0; i<=RECODE_BLOCKS ; i++) {\
			R[i] = (unsigned int)isprng();\
			for (j=0; j<ATT_PER_BLOCK ; j++){\
				if ( (i*ATT_PER_BLOCK+j)<NUM_CONDITIONS ) {\
					tmpAtt = GET_ATT(R[i],j);\
					switch (tmpAtt) {\
						case 0: SET_ATT(R[i],3,j); break;\
					}\
				}\
				else\
					SET_ATT(R[i],0,j);\
			}\
		}\
	}

// Soc aqui la cosa es que necessita imprimir be

#define PRINT_RULE(STREAM,R) {\
	int i,j,tmp;\
	for (i=0;i<=RECODE_BLOCKS;i++){\
		for (j=0;j<ATT_PER_BLOCK && (i*ATT_PER_BLOCK+j)<NUM_CONDITIONS;j++){\
			tmp = GET_ATT(R[i],j);\
			switch (tmp) {\
				case 1: fprintf(STREAM,"0"); break;\
				case 2: fprintf(STREAM,"1"); break;\
				default: fprintf(STREAM,"#"); break;\
			}\
		}\
	}\
}

int isRuleMatched ( RULE rule, INSTANCE ins ) {
	register int i,iFlag;
	
	for ( i=0, iFlag=1 ; i<=RECODE_BLOCKS /*&& iFlag*/ ; i++) 
		if ( (rule[i]&ins[i]) != ins[i] )
			iFlag = 0;
			
	return iFlag;
	
}


#else
// The default character implementation 
#define RULE char*

#define NEW_RULE ((char *)malloc(NUM_CONDITIONS*sizeof(char)))

#define DELETE_RULE(A) free(A)

#define RANDOM_INIT_RULE(R) {\
		int iRIR;\
		for(iRIR=0; iRIR<NUM_CONDITIONS ; iRIR++)\
			switch ( (isprng()%4UL) ) {\
				case 0UL: R[iRIR] = '0'; break;\
				case 1UL: R[iRIR] = '1'; break;\
				default : R[iRIR] = '#'; break;\
			}\
	}
	
#define PRINT_RULE(STREAM,R) {\
		int iPR;\
		for (iPR=0;iPR<NUM_CONDITIONS;iPR++)\
			fprintf(STREAM,"%c",R[iPR]);\
	}
	
int isRuleMatched ( RULE r, INSTANCE i ) {
		int iMRCL,iFlag;
		
		for ( iMRCL=0, iFlag=1 ; iMRCL<NUM_CONDITIONS /*&& iFlag*/ ; iMRCL++ )
			if ( i[iMRCL]!=r[iMRCL] && r[iMRCL]!='#' )
				iFlag=0;
		
		return iFlag;
}

#endif 
#endif

#endif
