/*
 * 
 *  Copyright (C) Xavier Llor
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 */

#ifndef _RULESET_
#define _RULESET_
 
#include "definitions.h" 
#include "rule.h"

#ifdef RULESET_LIST
// The default implementation of a rule set using a list
#define RULESET RULE*

#define NEW_RULESET ((RULESET)malloc(NUM_RULES*sizeof(RULE)))

#define INIT_RULESET(RS) {\
	int iIRS;\
	for ( iIRS=0 ; iIRS<NUM_RULES ; iIRS++ ){\
		RS[iIRS] = NEW_RULE;\
		assert(RS[iIRS]);\
	}\
} 

#define RANDOMIZE_RULESET(RS) {\
	int iRRS;\
	for ( iRRS=0 ; iRRS<NUM_RULES ; iRRS++ )\
		RANDOM_INIT_RULE(RS[iRRS]);\
} 

#define PRINT_RULESET(STREAM,RS) {\
	int iPRS;\
	for ( iPRS=0 ; iPRS<NUM_RULES ; iPRS++ ){\
		PRINT_RULE(STREAM,RS[iPRS]);\
		fprintf(STREAM,"\n");\
	}\
} 

#define DELETE_RULESET(RS) {\
	int iDRS;\
	for ( iDRS=0 ; iDRS<NUM_RULES ; iDRS++ )\
		free(RS[iDRS]);\
	free(RS);\
}

#define GET_RULE_FROM_RULESET(RS,I) RS[I]

int countMatchedRules ( RULESET rs, INSTANCE i ) {
	long lCnt,lAcc;
	
	for ( lCnt=lAcc=0L ; lCnt<NUM_RULES ; lCnt++ )
		if ( isRuleMatched(rs[lCnt],i) )
			lAcc++;
			
	return lAcc;
}
#endif

#endif





